/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.data;

import amberstone.lang.KeyFunction;
import java.math.BigInteger;

public class CompoundKey implements Key, KeyFunction
{
	public boolean clone = true;
	public String DELIM = "::";
	private int type = 5555;
	protected KeyFunction[] keys;
	private int index =-1;
	private int size;
	private static int[] m = {100129,100313,100213,100151,100153,100169,100183,100189,100193,100207,100213,100237,100267,100271,100279,100291,100297,100333,100343};

	public CompoundKey()
	{
	}

	public CompoundKey(int size)
	{
		this.size = size;
		keys = new KeyFunction[size];
	}

	public CompoundKey(KeyFunction[] keys)
	{
		this.keys = keys;
		this.size = this.index = keys.length -1;
	}

	public void setIndex(int i)
	{
		this.index = i;
	}

	public void reset()
	{
		this.index = -1;
	}

	public int keyCount()
	{
		return index+1;
	}

	public void append(KeyFunction key)
	{
		keys[++index] = key;
	}

	public int hashCode()
	{
		int h = 0;

		try
		{
			for(int i=0; i<=index; i++)
			{
				h += keys[i].getKey().hashCode()*m[i];
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}

		return h & 0x7FFFFFFF;
	}

	public boolean isNull()
	{
		return index == -1;
	}

	public boolean equalsKey(Key key)
	{
		try
		{
			CompoundKey ckey = (CompoundKey)key;

			if(ckey.index == index)
			{
				KeyFunction[] ckeys = ckey.keys;

				for(int i=0; i<=index; i++)
				{
					Key key1 = keys[i].getKey();
					Key key2 = ckeys[i].getKey();
					if(!key1.equalsKey(key2))
					{
						return false;
					}
				}
			}
			else
			{
				return false;
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}

		return true;
	}

	public Key cloneKey()
	{
		if(!clone) return this;

		CompoundKey clone = new CompoundKey();

		try
		{
			KeyFunction[] klones = new KeyFunction[index+1];
			for(int i=0; i<=index; i++)
			{
				klones[i] = (KeyFunction)(keys[i].getKey().cloneKey());
			}

			clone.keys = klones;
			clone.index = index;
			clone.DELIM = DELIM;
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		return clone;
	}

	public ByteArray byteValue()
	{
		return null;
	}

	public Key getKey()
	{
		return this;
	}

	public KeyFunction[] getKeys()
	{
		return this.keys;
	}

	public int getType()
	{
		return this.type;
	}


	public int intValue()
	{
		return index+1;
	}

	public long longValue()
	{
		return -1l;
	}

	public String toString()
	{
		StringBuilder builder = new StringBuilder();
		try
		{
			for(int i=0; i<=index; i++)
			{
				if(i>0)
				{
					builder.append(DELIM);
				}
				builder.append(keys[i].getKey().toString());
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}

		return builder.toString();
	}
}