
/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.data.fields;

import amberstone.data.*;
import amberstone.lang.*;
import amberstone.dictionary.*;

public abstract class Int extends Field implements Key, KeyFunction
{
	protected IntFunction loadFunc;
	protected BytesFunction bytesFunc;
	protected int intValue;
	protected int hashCode;
	protected Dictionary dictionary;

	public Int(int type,
			   String name,
			   int index,
			   IntFunction loadFunc)
	{
		super(type, name, index);
		this.loadFunc = loadFunc;
	}

	public void setDictionary(Dictionary dictionary)
	{
		this.dictionary = dictionary;
	}

	public Dictionary getDictionary()
	{
		return this.dictionary;
	}

	public void clear()
	{
		this.intValue = this.hashCode = 0;
	}

	public void setLoadFunction(IntFunction loadFunc)
	{
		this.loadFunc = loadFunc;
		if(loadFunc instanceof BytesFunction)
		{
			bytesFunc = (BytesFunction)loadFunc;
		}
	}

	public boolean equalsKey(Key k)
	{
		return this.intValue == k.intValue();
	}

	public boolean copyKey(Key key)
	{
		return false;
	}

	public Key cloneKey()
	{
		IntKey klone = new IntKey();
		klone.setType(type);
		klone.setIntValue(intValue);
		klone.setHashCode(hashCode());
		klone.setDictionary(this.dictionary);
		return klone;
	}

	public int getType()
	{
		return this.type;
	}

	public void setIntValue(int intValue)
	{
		this.intValue = intValue;
	}

	//Key interface

	public int intValue()
	{
		return this.intValue;
	}

	public long longValue()
	{
		return (long)this.intValue;
	}

	public ByteArray byteValue()
	{
		return null;
	}

	public boolean isNull()
	{
		return false;
	}

	//Key Function interface

	public abstract Key getKey();

	//IntFunction interface

	public int getInt(Function caller)
	{
		return this.intValue;
	}

	//LongFunction interface
	public long getLong(Function caller)
	{
		return (long)this.intValue;
	}

	public int getTime(Function caller) throws FunctionException
	{
		throw new FunctionException("Time not supported for int type.", caller);
	}

	public ByteArray getBytes(Function caller) throws FunctionException
	{
		throw new FunctionException("Bytes not supported for int type.", caller);
	}

	public String signiture()
	{
		return this.name+"."+maxLength();
	}

	public String toString()
	{
		if(dictionary == null)
		{
			return Integer.toString(intValue);
		}
		else
		{
			try
			{
				ByteArray b = dictionary.getTerm(intValue);
				return b.toString();
			}
			catch(Exception e)
			{
				System.out.println("Error:"+intValue);
				e.printStackTrace();
			}
		}

		return null;
	}
}