/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.data.fields;

import amberstone.data.*;
import amberstone.lang.*;

public class TimeInt extends Int4
{
	private static int t = 435345355;
	private DeflateTime deflateTime;
	private InflateTime inflateTime;

	public TimeInt(String name,
				   int index,
				   IntFunction loadFunc)
	{
		super(t, name, index, loadFunc);
	}


	public int prepareWrite() throws FieldException
	{
		try
		{
			ByteArray b = this.bytesFunc.getBytes();
			this.intValue = this.deflateTime.getTime(b);
		}
		catch(Exception e)
		{
			throw new FieldException(this.name, e);
		}

		if(buffer.getPointer()+4 < buffer.getBuffer().length)
		{
			return 4;
		}
		else
		{
			return -1;
		}
	}

	public void setTemplate(String template) throws Exception
	{
		deflateTime = new DeflateTime(template);
		inflateTime = new InflateTime(template);
	}

	public Field cloneField()
	{
		return new TimeInt(this.name, this.columnIndex, this.loadFunc);
	}

	public int getTime(Function caller)
	{
		return getInt(caller);
	}

	public String toString()
	{
		try
		{
			return inflateTime.getBytes(intValue).toString();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}

		return null;
	}
}