/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.dictionary;

import amberstone.lang.*;


import java.util.Map;

public class DictionaryFactory extends Function implements DictionaryFunction
{

	private static Map<String, Dictionary> dictionaries;
	private Dictionary dictionary;

	private static final FunctionType[][] allow = {{FunctionType.STRING_LITERAL, FunctionType.NUMBER_LITERAL}};

	public FunctionType[][] strict()
	{
		return allow;
	}

	public void params(Function[] params) throws ParameterException
	{
		try
		{
			String name = params[0].toString();
			Dictionary d = this.dictionaries.get(name);

			if(d == null)
			{
				d = new DiskDictionary(name);
				((DiskDictionary)d).open();
				this.dictionaries.put(name, d);
			}

			int cacheSize = ((NumberLiteral)params[1]).getInt();
			this.dictionary = new CacheDictionary(d, cacheSize);
		}
		catch(Exception e)
		{
			throw new ParameterException(this, "", e);
		}
	}

	public static void setDictionaries(Map<String, Dictionary> _dictionaries)
	{
		dictionaries = _dictionaries;
	}

	public Dictionary getDictionary()
	{
		return this.dictionary;
	}
}