/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.dictionary;

import java.io.IOException;
import java.util.Map;
import java.util.HashMap;

import amberstone.lang.DictionaryFunction;
import amberstone.data.ByteArray;
import amberstone.resources.ResourceIterator;
import amberstone.resources.MemoryResourceIterator;
import amberstone.util.MutableInteger;

public class MemoryDictionary implements Dictionary,
										 DictionaryFunction
{
	private Map<ByteArray, MutableInteger> terms;
	private Map<MutableInteger, ByteArray> ids;
	private MutableInteger ikey = new MutableInteger();
	private Dictionary dictionary;

	public MemoryDictionary(Dictionary dictionary) throws Exception
	{
		this.dictionary = dictionary;
		this.terms = new HashMap();
		this.ids = new HashMap();
		ResourceIterator it = dictionary.terms();
		for(ByteArray term = it.next(); term != null; term = it.next())
		{
			int i = dictionary.putTerm(term);
			MutableInteger mi = new MutableInteger(i);
			terms.put(term, mi);
			ids.put(mi, term);
		}
	}

	public void clear()
	{
		this.terms = null;
		this.ids = null;
	}

	public String getName()
	{
		return this.dictionary.getName();
	}

	public Dictionary getDictionary()
	{
		return this;
	}

	public ResourceIterator terms() throws IOException
	{
		return new MemoryResourceIterator(terms.keySet().iterator());
	}

	public int putTerm(ByteArray term)
		throws IOException
	{
		MutableInteger id =terms.get(term);

		if(id == null)
		{
			return 0;
		}
		else
		{
			return id.getInt();
		}
	}

	public ByteArray getTerm(int id) throws Exception
	{
		ikey.setInt(id);
		return ids.get(ikey);
	}
}

