/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.dictionary;

import amberstone.data.ByteArray;
import amberstone.resources.Resource;
import amberstone.resources.ResourceIterator;
import amberstone.util.MutableInteger;
import amberstone.util.LRUCache;
import java.util.Map;
import java.util.HashMap;
import java.io.IOException;

public class ResourceDictionary implements Dictionary
{
	private Resource resource;
	private MutableInteger ikey = new MutableInteger();
	private ByteArray notFound = new ByteArray("Key Not Found");

	public ResourceDictionary(Resource resource)
	{
		this.resource = resource;
	}

	public String getName()
	{
		return this.resource.getName();
	}

	public ResourceIterator terms() throws IOException
	{
		return resource.keys();
	}

	public int putTerm(ByteArray term)
		throws IOException
	{
		ByteArray bytes = resource.getValue(term);
		if(bytes == null)
		{
			return 0;
		}
		else
		{
			return bytes.parseInt();
		}
	}

	public ByteArray getTerm(int id)
		throws IOException
	{
		 ByteArray ba = resource.getKey(new ByteArray(Integer.toString(id)));
		if(ba == null)
		{
			return notFound;
		}
		else
		{
			return ba;
		}
	}
}