/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.lang;

import amberstone.data.*;

import java.util.Set;

public class Col extends Function implements IntFunction,
										     LongFunction,
										     Dimension,
											 BytesFunction,
											 TimeFunction,
											 Nullable
{
	private static DataRecord dataRecord;
	private static Set activeFields;
	public static boolean inContext;
	private String dimension;

	private Nullable nullable;
	private Field field;
	private String name;

	private static final FunctionType[][] allow = {{FunctionType.STRING_LITERAL}};

	public static void setDataRecord(DataRecord _dataRecord)
	{
		dataRecord = _dataRecord;
	}

	public static void setActiveFields(Set _activeFields)
	{
		activeFields = _activeFields;
	}

	public static Set getActiveFields()
	{
		return activeFields;
	}

	public FunctionType[][] strict()
	{
		return allow;
	}

	public String getDimension()
	{
		return this.dimension;
	}

	public String getName()
	{
		return name;
	}

	public void params(Function[] params) throws ParameterException
	{
		if(!inContext)
		{
			throw new ParameterException(params[0], "Field is not in context.");
		}

		String fieldName = dimension = params[0].toString();
		this.name = fieldName;
		Object o = dataRecord.getField(fieldName);
		if(o == null)
		{
			throw new ParameterException(params[0], "The field '"+fieldName+"' is undefined.");
		}

		this.field = (Field)o;
		this.nullable = (Nullable)o;
		activeFields.add(this.field);
	}

	public int getInt() throws FunctionException
	{
		return this.field.getInt(this);

	}

	public long getLong() throws FunctionException
	{
		return this.field.getLong(this);
	}

	public Key getKey()
	{
		return this.field.getKey();
	}

	public ByteArray getBytes() throws FunctionException
	{
		return this.field.getBytes(this);
	}

	public int getTime() throws FunctionException
	{
		return this.field.getTime(this);
	}

	public boolean isNull()
	{
		return nullable.isNull();
	}

	public String toString()
	{
		return this.field.toString();
	}
}