/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.lang;

import amberstone.system.SystemException;

import java.util.List;
import java.io.PrintWriter;
import java.io.StringWriter;

public class CompilationException extends SystemException
{
	private String statement;
	private String underline;
	private List<String> tokens;
	private int tokenIndex;
	private String message;

	public CompilationException(int tokenIndex,
							    String message)
	{
		super("");
		this.tokenIndex = tokenIndex;
		this.message = message;
	}

	public CompilationException(int tokenIndex,
								String message,
								Exception cause)
	{
		super("", cause);
		this.message = message;
		this.tokenIndex = tokenIndex;
	}

	public CompilationException(int tokenIndex,
								Exception cause)
	{
		super("", cause);
		this.tokenIndex = tokenIndex;
	}

	public void setTokens(List<String> tokens)
	{
		this.tokens = tokens;
	}

	public void setStatement(String statement)
	{
		this.statement = statement;
	}

	public void setFunctionUsage()
	{

	}

	public String getMessage()
	{
		try
		{
			StringWriter st = new StringWriter();
			PrintWriter printer = new PrintWriter(st);
			printer.print("-Compilation error: ");
			if(message != null)
			{
				printer.print(message);
			}
			printer.println("");


			if(tokens != null)
			{
				printer.println(getErrorStatement());
				printer.println(this.underline);
			}
			else
			{
				printer.println(statement);
			}

			return st.toString();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}

		return null;
	}


	private String getErrorStatement()
	{
		StringBuilder buf = new StringBuilder();
		int index = 0;

		for(int i=0; i<tokens.size(); i++)
		{
			if(i == tokenIndex)
			{
				index = buf.length();
				buf.append(" ");
			}
			buf.append(tokens.get(i));
		}


		StringBuilder ubuf = new StringBuilder();
		for(int i=0; i<index; i++)
		{
			ubuf.append("-");
		}
		ubuf.append("^");
		this.underline = ubuf.toString();

		return buf.toString();
	}
}