/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.lang;

import amberstone.data.ByteArray;
import amberstone.load.DelimitedRecord;
import amberstone.load.DelimitedField;

import java.util.Set;

public class DataColumn extends Function implements IntFunction, LongFunction, BytesFunction
{
	private static DelimitedRecord delimitedRecord;
	private static Set activeDelimitedFields;

	private DelimitedField delimitedField;

	private static final FunctionType[][] allow = {{FunctionType.NUMBER_LITERAL}};

	public FunctionType[][] strict()
	{
		return allow;
	}

	public static void setDelimitedRecord(DelimitedRecord _delimitedRecord)
	{
		delimitedRecord = _delimitedRecord;
	}

	public static void setActiveDelimitedFields(Set _activeDelimitedFields)
	{
		activeDelimitedFields = _activeDelimitedFields;
	}

	public void params(Function[] params) throws ParameterException
	{
		int column = ((NumberLiteral)params[0]).getInt();

		Object o = delimitedRecord.getField(column);

		if(o == null)
		{
			throw new ParameterException(params[0], "Data field "+column+" is undefined.");
		}

		this.delimitedField = (DelimitedField)o;
		this.activeDelimitedFields.add(this.delimitedField);
	}

	public int getInt() throws FunctionException
	{
		return this.delimitedField.getInt();
	}

	public long getLong() throws FunctionException
	{
		return this.delimitedField.getLong();
	}

	public ByteArray getBytes() throws FunctionException
	{
		return this.delimitedField.getBytes();
	}
}