/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.lang;

import amberstone.data.Key;
import amberstone.data.ByteArray;

public class Echo extends Function implements KeyFunction, IntFunction, LongFunction, BytesFunction, TimeFunction
{
	private KeyFunction keyFunc;
	private IntFunction intFunc;
	private LongFunction longFunc;
	private BytesFunction bytesFunc;
	private TimeFunction timeFunc;
	protected String message;

	private static final FunctionType[][] allow = {{FunctionType.KEY_FUNCTION},
												   {FunctionType.INT_FUNCTION},
												   {FunctionType.LONG_FUNCTION},
												   {FunctionType.BYTES_FUNCTION},
												   {FunctionType.TIME_FUNCTION},
												   {FunctionType.STRING_LITERAL, FunctionType.KEY_FUNCTION },
												   {FunctionType.STRING_LITERAL, FunctionType.DATE_FUNCTION },
												   {FunctionType.STRING_LITERAL, FunctionType.TIME_FUNCTION },
												   {FunctionType.STRING_LITERAL, FunctionType.INT_FUNCTION },
												   {FunctionType.STRING_LITERAL, FunctionType.LONG_FUNCTION},
												   {FunctionType.STRING_LITERAL, FunctionType.BYTES_FUNCTION},
												   {FunctionType.STRING_LITERAL, FunctionType.CELL_FUNCTION}};

	public FunctionType[][] strict()
	{
		return allow;
	}

	public void params(Function[] params)
	{
		int funcIndex = 0;

		if(params.length == 2)
		{
			message = params[0].toString();
			funcIndex = 1;
		}

		if(params[funcIndex] instanceof KeyFunction)
		{
			keyFunc = (KeyFunction)params[funcIndex];
		}

		if(params[funcIndex] instanceof IntFunction)
		{
			intFunc = (IntFunction)params[funcIndex];
		}

		if(params[funcIndex] instanceof LongFunction)
		{
			longFunc = (LongFunction)params[funcIndex];
		}

		if(params[funcIndex] instanceof BytesFunction)
		{
			bytesFunc = (BytesFunction)params[funcIndex];
		}


		if(params[funcIndex] instanceof TimeFunction)
		{
			timeFunc = (TimeFunction)params[funcIndex];
		}
	}

	public Key getKey()  throws FunctionException
	{
		Key key = keyFunc.getKey();
		echo(key.toString());
		return key;
	}

	public int getInt()  throws FunctionException
	{
		int i = intFunc.getInt();
		echo(Integer.toString(i));
		return i;
	}

	public int getTime() throws FunctionException
	{
		int i = timeFunc.getTime();
		echo(Integer.toString(i));
		return i;
	}

	public long getLong()  throws FunctionException
	{
		long l = longFunc.getLong();
		echo(Long.toString(l));
		return l;
	}

	public ByteArray getBytes()  throws FunctionException
	{
		ByteArray bytes = bytesFunc.getBytes();
		echo(bytes.toString());
		return bytes;
	}

	protected void echo(String s)
	{
		if(message != null)
		{
			System.out.print("  "+ message+":");
		}

		System.out.print("'"+s+"'");
		ln();
	}

	protected void ln()
	{

	}
}