/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.lang;

import amberstone.data.ByteArray;
import amberstone.util.MurmurHash3;

public class Hash extends Function implements IntFunction, LongFunction
{
	private BytesFunction input;
	private ByteArray byteArray = new ByteArray();
	private byte[] hash = new byte[16];

	private FunctionType[][] allow = {{FunctionType.BYTES_FUNCTION}};

	public FunctionType[][] strict()
	{
		return allow;
	}

	public void params(Function[] params) throws ParameterException
	{
		this.input = (BytesFunction)params[0];
	}

	public long getLong() throws FunctionException
	{
		try
		{
			return getLong(this.input.getBytes());
		}
		catch(Exception e)
		{
			throw new FunctionException("Error creating hash.", this, e);
		}
	}

	public int getInt() throws FunctionException
	{
		try
		{
			return getInt(this.input.getBytes());
		}
		catch(Exception e)
		{
			throw new FunctionException("Error creating hash.", this, e);
		}
	}

	public long getLong(ByteArray bytes)
	{
		return MurmurHash3.MurmurHash3_x64_64(bytes.toByteArray(), 1234123423);
	}

	public int getInt(ByteArray bytes)
	{
		return MurmurHash3.MurmurHash3_x64_32(bytes.toByteArray(), 1234123423);
	}

}


