/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.lang;

import amberstone.data.Key;
import amberstone.data.ByteArray;
import amberstone.data.IntKey;
import amberstone.data.StringKey;


public class If extends Function implements KeyFunction, IntFunction, LongFunction, BytesFunction
{
	private BooleanFunction boolFunc;

	private LongFunction longFunc1;
	private LongFunction longFunc2;

	private BytesFunction bytesFunc1;
	private BytesFunction bytesFunc2;

	private IntFunction intFunc1;
	private IntFunction intFunc2;

	private KeyFunction keyFunc1;
	private KeyFunction keyFunc2;

	private static final NumberLiteral zero = new NumberLiteral(0D);
	private static final ByteArray nullBytes = new ByteArray("");

	private static final FunctionType[][] allow = {{FunctionType.BOOLEAN_FUNCTION, FunctionType.INT_FUNCTION},
						                           {FunctionType.BOOLEAN_FUNCTION, FunctionType.KEY_FUNCTION},
	                                               {FunctionType.BOOLEAN_FUNCTION, FunctionType.LONG_FUNCTION},
	                                               {FunctionType.BOOLEAN_FUNCTION, FunctionType.BYTES_FUNCTION},
	                                               {FunctionType.BOOLEAN_FUNCTION, FunctionType.BYTES_FUNCTION, FunctionType.BYTES_FUNCTION},
	                                               {FunctionType.BOOLEAN_FUNCTION, FunctionType.LONG_FUNCTION, FunctionType.LONG_FUNCTION},
												   {FunctionType.BOOLEAN_FUNCTION, FunctionType.INT_FUNCTION, FunctionType.INT_FUNCTION},
												   {FunctionType.BOOLEAN_FUNCTION, FunctionType.KEY_FUNCTION, FunctionType.KEY_FUNCTION}};

	public FunctionType[][] strict()
	{
		return allow;
	}

	public void params(Function[] params)
	{
		this.boolFunc = (BooleanFunction)params[0];

		if(params[1] instanceof IntFunction)
		{
			this.intFunc1 = (IntFunction)params[1];
			if(params.length == 3)
			{
				this.intFunc2 = (IntFunction)params[2];
			}
			else
			{
				this.intFunc2 = zero;
			}
		}

		if(params[1] instanceof BytesFunction)
		{
			this.bytesFunc1 = (BytesFunction)params[1];
			if(params.length == 3)
			{
				this.bytesFunc2 = (BytesFunction)params[2];
			}
			else
			{
				this.bytesFunc2 = nullBytes;
			}
		}

		if(params[1] instanceof LongFunction)
		{
			this.longFunc1 = (LongFunction)params[1];
			if(params.length == 3)
			{
				this.longFunc2 = (LongFunction)params[2];
			}
			else
			{
				this.intFunc2 = zero;
			}
		}

		if(params[1] instanceof KeyFunction)
		{
			this.keyFunc1 = (KeyFunction)params[1];
			if(params.length == 3)
			{
				this.keyFunc2 = (KeyFunction)params[2];
			}
			else
			{
				this.keyFunc2 = new StringLiteral("");
			}
		}
	}

	public int getInt() throws FunctionException
	{
		if(this.boolFunc.getBoolean())
		{
			return this.intFunc1.getInt();
		}
		else
		{
			return this.intFunc2.getInt();
		}
	}

	public ByteArray getBytes() throws FunctionException
	{
		if(this.boolFunc.getBoolean())
		{
			return this.bytesFunc1.getBytes();
		}
		else
		{
			return this.bytesFunc2.getBytes();
		}
	}

	public long getLong() throws FunctionException
	{
		if(this.boolFunc.getBoolean())
		{
			return this.longFunc1.getLong();
		}
		else
		{
			return this.longFunc2.getLong();
		}
	}

	public Key getKey() throws FunctionException
	{
		if(this.boolFunc.getBoolean())
		{
			return this.keyFunc1.getKey();
		}
		else
		{
			return this.keyFunc2.getKey();
		}
	}
}