/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.lang;

import amberstone.data.*;
import amberstone.report.*;
import amberstone.dictionary.*;
import amberstone.util.KeyValuePair;

public class Inflate extends Function implements BytesFunction, KeyFunction
{
	private IntFunction intFunc;
	private DictionaryFunction dictionaryFunc;

	private FunctionType[][] allow = {{FunctionType.INT_FUNCTION, FunctionType.DICTIONARY_FUNCTION}};

	public FunctionType[][] strict()
	{
		return allow;
	}

	public void params(Function[] params) throws ParameterException
 	{
		this.intFunc = (IntFunction)params[0];
		this.dictionaryFunc = (DictionaryFunction)params[1];
	}

	public ByteArray getBytes() throws FunctionException
	{
		try
		{
			Dictionary dictionary = dictionaryFunc.getDictionary();
			int id = intFunc.getInt();
			ByteArray bytes = dictionary.getTerm(id);
			if(bytes == null)
			{
				return new ByteArray("");
			}

			return bytes;
		}
		catch(Exception e)
		{
			throw new FunctionException("Error inflating key.", this, e);
		}
	}

	public Key getKey() throws FunctionException
	{
		return new BytesKey(getBytes());
	}
}