/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.lang;

import amberstone.data.ByteArray;
import amberstone.resources.*;
import amberstone.dictionary.*;

import java.io.*;

public class IpDictionary extends Function implements DictionaryFunction, Dictionary
{

	private static final FunctionType[][] allow = {};

	public FunctionType[][] strict()
	{
		return allow;
	}

	public String getName()
	{
		return "IP";
	}

	public Dictionary getDictionary()
	{
		return this;
	}

	public ResourceIterator terms() throws IOException
	{
		return null;
	}

	public ByteArray getTerm(int i)
	{
		StringBuilder buf = new StringBuilder();
		buf.append(getFirstSegment(i)).append(".");
		buf.append(getSecondSegment(i)).append(".");
		buf.append(getThirdSegment(i)).append(".");
		buf.append(getFourthSegment(i));
		return new ByteArray(buf.toString());
	}

	public int putTerm(ByteArray bytes) throws Exception
	{
		if(bytes.isNull())
		{
			return 0;
		}

		byte[] buffer = bytes.getBuffer();
		int offset = bytes.getOffset();
		int numBytes = bytes.getLength();

		int segment1 = 0;
		int segment2 = 0;
		int segment3 = 0;
		int segment4 = 0;
		int segmentNumber = 4;
		int num = 0;
		int power=1;

		for(int i=offset+numBytes-1; i>=offset; i--)
		{
			byte b = buffer[i];
			if(b > 47 && b < 58)
			{
				num = num+((b-48)*power);
				power = power*10;
			}
			else if(b == 46)
			{
				if(segmentNumber == 4)
				{
					segment4 = num;
				}
				else if(segmentNumber == 3)
				{
					segment3 = num;
				}
				else
				{
					segment2 = num;
				}

				num = 0;
				segmentNumber--;
				power = 1;
			}
			else
			{
				byte[] bad = {(byte)b};
				throw new Exception("Malformed IP address, bad byte:"+(char)b);
			}

			segment1 = num;
		}

		return (segment1<<24) + (segment2<<16) + (segment3<<8)+ segment4;
	}

	private int getFirstSegment(int value)
	{
		return (((value & 0xFF000000) >>>  24) & 0xFF);
	}

	private int getSecondSegment(int value)
	{
		return (((value & 0x00FF0000) >>>  16) & 0xFF);
	}

	private int getThirdSegment(int value)
	{
		return (((value & 0x0000FF00) >>>  8) & 0xFF);
	}

	private int getFourthSegment(int value)
	{
		return (value & 0x000000FF);
	}
}