/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.lang;

import amberstone.resources.Resource;
import amberstone.data.ByteArray;

public class ResourceMap extends Function implements BytesFunction
{
	private BytesFunction bytesFunc;
	private ResourceFunction resourceFunc;
	private BytesFunction defaultFunc;

	private static final FunctionType[][] allow = {{FunctionType.RESOURCE_FUNCTION, FunctionType.BYTES_FUNCTION},
	                                               {FunctionType.RESOURCE_FUNCTION, FunctionType.BYTES_FUNCTION, FunctionType.BYTES_FUNCTION}};

	public FunctionType[][] strict()
	{
		return this.allow;
	}

	public void params(Function[] params)
	{
		this.resourceFunc = (ResourceFunction)params[0];
		this.bytesFunc = (BytesFunction)params[1];
		if(params.length == 3)
		{
			this.defaultFunc = (BytesFunction)params[2];
		}
	}

	public ByteArray getBytes() throws FunctionException
	{
		try
		{
			Resource resource = resourceFunc.getResource();
			ByteArray bytes = bytesFunc.getBytes();
			ByteArray value = resource.getValue(bytes);
			if(value == null)
			{
				return defaultFunc.getBytes();
			}

			return value;
		}
		catch(Exception e)
		{
			throw new FunctionException("Resource map error.", this, e);
		}
	}
}