/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.lang;

import amberstone.data.ByteArray;

public class TrimRight extends Function implements BytesFunction, IntFunction, LongFunction
{
	private BytesFunction sourceBytesFunc;
	private ByteArray trimmed = new ByteArray();

	private static final FunctionType[][] allow = {{ FunctionType.BYTES_FUNCTION }};

	public FunctionType[][] strict()
	{
		return allow;
	}

	public void params(Function[] params) throws ParameterException
	{
		this.sourceBytesFunc  = (BytesFunction)params[0];
	}

	public ByteArray getBytes() throws FunctionException
	{
		ByteArray sourceBytes = this.sourceBytesFunc.getBytes();
		trimmed.set(sourceBytes.getBuffer(), sourceBytes.getOffset(), sourceBytes.getLength());
		trimmed.rtrim();
		return trimmed;
	}

	public int getInt() throws FunctionException
	{
		return this.getBytes().parseInt();
	}

	public long getLong() throws FunctionException
	{
		return this.getBytes().parseLong();
	}
}