/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.lang;

import amberstone.data.*;

public class Trio extends Function implements KeyFunction, Key, Dimension
{
	private KeyFunction keyFunc1;
	private KeyFunction keyFunc2;
	private KeyFunction keyFunc3;

	private Key key1;
	private Key key2;
	private Key key3;

	private static FunctionType[][] allow = {{FunctionType.KEY_FUNCTION, FunctionType.KEY_FUNCTION, FunctionType.KEY_FUNCTION}};

	public FunctionType[][] strict()
	{
		return allow;
	}

	public Trio(Key k1, Key k2, Key k3)
	{
		this.keyFunc1 = (KeyFunction)k1;
		this.keyFunc2 = (KeyFunction)k2;
		this.keyFunc3 = (KeyFunction)k3;

		key1 = k1;
		key2 = k2;
		key3 = k3;
	}

	public Trio()
	{
		super();
	}

	public int getType()
	{
		return 232323;
	}

	public boolean isNull()
	{
		return false;
	}

	public void params(Function[] params)
	{
		this.keyFunc1 = (KeyFunction)params[0];
		this.keyFunc2 = (KeyFunction)params[1];
		this.keyFunc3 = (KeyFunction)params[2];
	}

	public Key getKey() throws FunctionException
	{
		this.key1 = keyFunc1.getKey();
		this.key2 = keyFunc2.getKey();
		this.key3 = keyFunc3.getKey();
		return this;
	}

	public int hashCode()
	{
		return ((this.key1.hashCode()*1213123)+(this.key2.hashCode()*34567)+(this.key3.hashCode()*777777)) & 0x7FFFFFFF;
	}

	public boolean equalsKey(Key key)
	{
		Trio d = (Trio)key;
		return key1.equalsKey(d.key1) && key2.equalsKey(d.key2) && key3.equalsKey(d.key3);
	}

	public Key cloneKey()
	{
		return new Trio(key1.cloneKey(), key2.cloneKey(), key3.cloneKey());
	}

	public int intValue()
	{
		return -1;
	}

	public long longValue()
	{
		return -1L;
	}

	public ByteArray byteValue()
	{
		return null;
	}

	public String getDimension()
	{
		return "";
	}

	public String toString()
	{
		return key1.toString()+"::"+key2.toString()+"::"+key3.toString();
	}
}