/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.lang;

import amberstone.system.SystemException;

import java.util.List;
import java.io.PrintWriter;
import java.io.StringWriter;

public class UsageException extends Exception
{
	private Function call;
	private String message;

	public UsageException(String message, Function call)
	{
		super();
		this.call = call;
		this.message = message;
	}

	public String getMessage()
	{
		try
		{
			StringWriter st = new StringWriter();
			PrintWriter printer = new PrintWriter(st);
			printer.println(message);
			printer.println("Function usage:");

			FunctionType[][] allowed = this.call.strict();

			for(int i=0; i<allowed.length; i++)
			{
				printer.print(this.call.getCallName()+"(");
				FunctionType[] types = allowed[i];
				for(int t=0; t<types.length; t++)
				{
					if(t>0)
					{
						printer.print(", ");
					}
					printer.print(types[t]);
				}
				printer.println(")");
			}


			return st.toString();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}

		return null;
	}
}