/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.load;

import amberstone.system.*;

import java.io.StringWriter;
import java.io.PrintWriter;

public class LoadException extends SystemException
{
	private int recordNumber;
	private String fileName;

	public LoadException(int recordNumber,
						 String fileName,
						 Exception cause)
	{
		super("Exception during load:", cause);
		this.recordNumber = recordNumber;
		this.fileName = fileName;
	}

	public LoadException(int recordNumber,
						 String fileName)
	{
		super("Exception during load:");
		this.recordNumber = recordNumber;
		this.fileName = fileName;
	}

	public void log(int logLevel, Logger logger)
	{
		logger.ln();
		logger.log(getMessage());
		Throwable c = getCause();

		int i = 1;

		while(c != null)
		{
			logger.log("-Cause:");
			logger.log(c.getMessage());
			c = c.getCause();
			i++;
		}
		if(logLevel > 1)
		{
			logger.ln();
			logger.log("=========================================");
			logger.log("Trace:");
			logger.log(this);
			logger.log("=========================================");
		}

		logger.ln();

	}

	public String getMessage()
	{
		try
		{
			StringWriter st = new StringWriter();
			PrintWriter printer = new PrintWriter(st);
			//printer.println(super.getMessage());
			printer.print("*Load error in file:"+fileName+", ");
			printer.print("record number:"+recordNumber);
			return st.toString();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}

		return null;
	}
}