/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.read;

import amberstone.lang.*;
import amberstone.data.Field;
import amberstone.file.FileStat;
import java.util.List;
import java.io.*;

public class Exporter implements Printer
{
	private String delim;
	private String outDir;
	private String lastName;
	private PrintWriter out = null;
	private int serialNumber;
	private int maxSize;
	private int currentSize;

	public Exporter(int maxSize, String delim, String outDir)
	{
		this.maxSize = maxSize;
		this.outDir = outDir;
		this.delim = delim;
	}

	public void open()
	{
		System.out.println("Exporting to directory:"+outDir);
	}

	public void close() throws Exception
	{
		if(out != null)
		{
			out.close();
		}
		System.out.println("Export complete");
	}

	public void print(Record record) throws Exception
	{
		String export = record.getExport();
		if(!export.equals(lastName))
		{
			if(out != null) out.close();
			lastName = export;
			serialNumber = 1;
			currentSize = 0;
			File file = getOutFile(export, serialNumber);
			out = new PrintWriter(new BufferedWriter(new FileWriter(file)));
		}

		StringBuilder buf = new StringBuilder();

		Col[] reads = record.getFields();

		for(int i=0; i<reads.length; i++)
		{
			if(i>0)
			{
				buf.append(delim);
			}

			Col read = reads[i];
			String value = read.toString();
			buf.append(value);
		}

		String line = buf.toString();

		if(line.length()+currentSize > maxSize)
		{
			if(out != null) out.close();
			File file = getOutFile(export, ++serialNumber);
			currentSize = 0;
			out = new PrintWriter(new BufferedWriter(new FileWriter(file)));
		}

		currentSize+=line.length();
		out.println(line);
	}

	private File getOutFile(String export, int serial)
	{
		File p = new File(outDir);
		p.mkdirs();
		File f = new File(p, export+"."+serial+".txt");
		return f;
	}
}