/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.read;

import amberstone.summary.Range;
import amberstone.summary.RangeFilter;

import java.io.*;
import java.util.*;

public class YearDir
{
	private RangeFilter filter;
	private Comparator comp;

	private File dir;

	public static List<YearDir> getYears(File root,
									     RangeFilter filter,
									     Comparator comp)
	{
		Set ranges = filter.getRanges();
		Iterator i = ranges.iterator();

		RangeFilter yearFilter = new RangeFilter();

		while(i.hasNext())
		{
			Range range = (Range)i.next();
			int beginYYYYMMDD = range.getBegin();
			int beginYYYY = (int)(beginYYYYMMDD/10000);

			int endYYYYMMDD   = range.getEnd();
			int endYYYY = (int)(endYYYYMMDD/10000);
			yearFilter.addRange(new Range(beginYYYY, endYYYY));
		}

		File[] dirs = root.listFiles(yearFilter);
		Arrays.sort(dirs, comp);
		List<YearDir> years = new ArrayList();
		for(int y=0; y<dirs.length; y++)
		{
			YearDir yearDir = new YearDir(dirs[y], filter, comp);
			years.add(yearDir);
		}

		return years;
	}

	public YearDir(File dir, RangeFilter filter, Comparator comp)
	{
		this.dir = dir;
		this.filter = filter;

		this.comp = comp;
	}

	public List<MonthDir> getMonths()
	{
		Set ranges = filter.getRanges();
		Iterator i = ranges.iterator();

		DirFilter monthFilter = new DirFilter(dir.getName());

		while(i.hasNext())
		{
			Range range = (Range)i.next();
			int beginYYYYMMDD = range.getBegin();
			int beginRange = (int)(beginYYYYMMDD/100);

			int endYYYYMMDD   = range.getEnd();
			int endRange = (int)(endYYYYMMDD/100);
			monthFilter.addRange(new Range(beginRange, endRange));
		}

		File[] dirs = dir.listFiles(monthFilter);

		Arrays.sort(dirs, comp);
		List<MonthDir> months = new ArrayList();
		for(int y=0; y<dirs.length; y++)
		{
			MonthDir monthDir = new MonthDir(dirs[y], filter, comp);
			months.add(monthDir);
		}

		return months;
	}

	public String toString()
	{
		return dir.getPath();
	}
}