/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.resources;

import amberstone.data.ByteArray;
import amberstone.util.LRUCache;

import java.io.IOException;

public class CacheResource implements Resource
{
	private Resource resource;
	private LRUCache cache;

	public CacheResource(Resource resource, int cacheSize)
	{
		this.resource = resource;
		this.cache  = new LRUCache(((int)(cacheSize*1.1)), 1.0F, cacheSize);
	}

	public void clear()
	{
		this.cache = null;
	}

	public String getName()
	{
		return resource.getName();
	}

	public ResourceIterator keys() throws IOException
	{
		return resource.keys();
	}

	public ResourceIterator values() throws IOException
	{
		return resource.values();
	}

	public String get(String key)
		throws IOException
	{
		ByteArray bytes = getValue(new ByteArray(key));
		if(bytes != null)
		{
			return bytes.toString();
		}

		return null;
	}

	public ByteArray getValue(ByteArray key)
		throws IOException
	{
		ByteArray bytes = (ByteArray)cache.get(key);
		if(bytes == null)
		{
			bytes = resource.getValue(key);
			if(bytes != null)
			{
				cache.put(key.cloneBytes(), bytes);
			}
		}

		return bytes;
	}

	public ByteArray getKey(ByteArray value)
		throws IOException
	{
		return resource.getKey(value);
	}
}