/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.resources;

import amberstone.dictionary.*;
import amberstone.lang.*;

import java.util.Map;

public class ResourceFactory extends Function implements ResourceFunction, DictionaryFunction
{
	private Resource resource;
	private Dictionary dictionary;
	private static Map<String, Resource> resources;

	public static void setResources(Map _resources)
	{
		resources = _resources;
	}

	private static final FunctionType[][] allow = {{ FunctionType.STRING_LITERAL, FunctionType.NUMBER_LITERAL }};

	public FunctionType[][] strict()
	{
		return allow;
	}

	public void params(Function[] params) throws ParameterException
	{
		String name = params[0].toString();
		this.resource = resources.get(name);
		if(this.resource == null)
		{
			throw new ParameterException(this, "The resource '"+name+"' is undefined");
		}

		this.dictionary = new ResourceDictionary(resource);
		int cacheSize = ((NumberLiteral)params[1]).getInt();
		this.dictionary = new CacheDictionary(this.dictionary, cacheSize);
	}

	public Resource getResource()
	{
		return this.resource;
	}

	public Dictionary getDictionary()
	{
		return this.dictionary;
	}
}