/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.summary;

public class PointQueue
{
	private Point head = new Point();
	private Point tail = new Point();

	public PointQueue()
	{
		head.back = tail;
		tail.forward = head;
	}

	public void clear()
	{
		head.back = tail;
		tail.forward = head;
	}

	public void remove(Point point)
	{
		unlink(point);
	}

	public Point removeLast()
	{
		if(tail.forward != head)
		{
			Point point = tail.forward;
			unlink(point);
			return point;
		}

		return null;
	}

	public Point removeFirst()
	{
		if(head.back != tail)
		{
			Point point = head.back;
			unlink(point);
			return point;
		}

		return null;
	}

	private void addBefore(Point point, Point before)
	{
		point.forward = before;
		point.back = before.back;
		before.back.forward = point;
		before.back = point;
	}

	private void addAfter(Point point, Point after)
	{
		point.back = after;
		point.forward = after.forward;
		after.forward.back = point;
		after.forward = point;
	}

	public void addFirst(Point point)
	{
		addBefore(point, head);
	}

	public void addLast(Point point)
	{
		addAfter(point, tail);
	}

	public void moveFirst(Point point)
	{
		point.back.forward = point.forward;
		point.forward.back = point.back;
		addFirst(point);
	}

	public void moveLast(Point point)
	{
		point.back.forward = point.forward;
		point.forward.back = point.back;
		addLast(point);
	}

	public void unlink(Point point)
	{
		point.back.forward = point.forward;
		point.forward.back = point.back;
		point.forward = null;
		point.back = null;
	}
}