/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.system;

import amberstone.dictionary.*;
import amberstone.resources.*;

import java.util.Map;
import java.util.HashMap;
import java.util.Properties;

import java.io.*;
import java.util.jar.*;
import java.util.zip.*;

public class AmberstoneConfig
{
	public static String AMBERSTONE_HOME;
	private Map<String, Resource> resources = new HashMap();
	private Properties lang;
	private Properties system;
	private Command command;
	private String dictionariesPath;
	private String resourcesPath;

	public Map<String, Resource> getResources()
	{
		return this.resources;
	}

	public AmberstoneConfig(Command command, Properties system, Properties lang)
		throws Exception
	{
		this.system = system;
		this.lang = lang;
		this.command = command;
		AMBERSTONE_HOME = System.getenv("AMBERSTONE_HOME");
		this.resourcesPath = AMBERSTONE_HOME + File.separatorChar + "resources";
		loadResources(resourcesPath);
	}

	public void loadResources(String path) throws Exception
	{
		ResourceFactory.setResources(this.resources);
		File dir = new File(path);
		File[] rs = dir.listFiles();
		for(int i=0; i<rs.length; i++)
		{
			if(rs[i].getName().endsWith(".txt"))
			{
				String name = rs[i].getName().split("\\.")[0];
				DiskResource diskResource = new DiskResource(resourcesPath, name);
				diskResource.load();
				this.resources.put(name, diskResource);
			}
		}
	}
}