/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.system;

import amberstone.file.*;
import amberstone.dictionary.Dictionary;
import amberstone.dictionary.DiskDictionary;
import java.io.File;
import java.io.FileFilter;
import java.util.Map;
import java.util.Iterator;

public class DeleteCommand implements Command
{
	private Config config;

	private String[] args;

	public DeleteCommand(String[] args)
	{
		this.args = args;
	}

	public boolean valid()
	{
		if(args.length == 4)
		{
			if(DateRange.valid(args[2], args[3]))
			{
				return true;
			}
			else
			{
				System.out.println("Invalid date range.");
				return false;
			}
		}
		else
		{
			return false;
		}
	}

	public String usage()
	{
		return "delete [table] [startYYYYMMDD] [endYYYYMMDD]";
	}

	public Selector getSelector()
	{
		return null;
	}

	public boolean needsResources()
	{
		return false;
	}

	public void run(Config config)
	{
		try
		{
			String table = args[1];
			int startYYYYMMDD = Integer.parseInt(args[2]);
			int endYYYYMMDD = Integer.parseInt(args[3]);
			delete(table, startYYYYMMDD, endYYYYMMDD);
		}
		catch(Exception e)
		{
			System.out.println("Error in delete command:");
			System.out.println(e.getMessage());
			Trace trace = new Trace();
			trace.trace(e, "delete", config.getLogsHome());
		}
	}

	private void delete(String table, int startYYYYMMDD, int endYYYYMMDD)
	{
		String root = Config.AMBERSTONE_HOME+File.separatorChar+"storage"+File.separatorChar+table+File.separatorChar+"table";
		File dir = new File(root);
		if(!dir.exists())
		{
			System.out.println("Table does not exist: "+table);
			return;
		}

		File[] years = dir.listFiles();
		for(int y=0; y<years.length;y++)
		{
			int YYYY = Integer.parseInt(years[y].getName());
			File[] months = years[y].listFiles();
			for(int m=0; m<months.length; m++)
			{
				int MM = Integer.parseInt(months[m].getName());

				File[] days = months[m].listFiles();
				for(int d=0; d<days.length; d++)
				{
					int DD = Integer.parseInt(days[d].getName());

					int YYYYMMDD = (YYYY*10000)+(MM*100)+DD;
					if(YYYYMMDD >= startYYYYMMDD && YYYYMMDD <= endYYYYMMDD)
					{
						delete(days[d]);
					}
				}
			}
		}
	}

	private void delete(File day)
	{
		System.out.println("Deleting:"+day.getPath());
		File[] blocks = day.listFiles();
		for(int b=0; b<blocks.length; b++)
		{
			File[] data = blocks[b].listFiles();
			for(int i=0; i<data.length; i++)
			{
				data[i].delete();
			}

			blocks[b].delete();
		}

		day.delete();
	}
}