/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.system;

import amberstone.load.Loader;

public class LoadCommand implements Command
{
	private boolean valid;
	private boolean archive;
	private String index;

	public LoadCommand(String args[])
	{
		if(args.length == 2)
		{
			this.index = args[1];
			valid = true;
		}
		else if(args.length == 3)
		{
			this.index = args[1];
			if(args[2].equals("archive"))
			{
				valid = true;
				archive = true;
			}
		}
		else
		{
			valid = false;
		}
	}

	public boolean valid()
	{
		return valid;
	}

	public String getIndex()
	{
		return this.index;
	}

	public boolean archive()
	{
		return this.archive;
	}

	public String usage()
	{
		return "load [index] <archive>";
	}

	public Selector getSelector()
	{
		return null;
	}

	public void run(Config config)
	{
		try
		{
			Loader loader = new Loader(config, archive);
			loader.load();
		}
		catch(Exception e)
		{
			System.out.println("Error in the loader:");
			System.out.println(e.getMessage());
			System.out.println("See loader.log for details.");
			Trace trace = new Trace();
			trace.trace(e, "loader", config.getLogsHome());
		}
	}

	public boolean needsResources()
	{
		return true;
	}
}