/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.system;

import amberstone.summary.Summarizer;

import java.io.*;

public class PartCommand implements Command
{
	private String clone;
	private String name;
	private boolean valid;

	public PartCommand(String args[])
	{
		if(args.length == 2 || args.length == 3)
		{
			valid = true;

			this.name = args[1];
			if(args.length == 3)
			{
				this.clone = args[2];
			}
		}
		else
		{
			valid = false;
		}
	}

	public boolean valid()
	{
		return valid;
	}


	public String usage()
	{
		return "part [name] <clone>";
	}

	public void run(Config config)
	{
		try
		{
			System.out.println("Adding new storage partition:"+name);

			String indexDir = Config.AMBERSTONE_HOME+File.separatorChar+"storage";

			String templateDir = null;
			if(clone != null)
			{
				templateDir = indexDir+File.separatorChar+clone+File.separatorChar+"conf";
			}
			else
			{
				templateDir = Config.AMBERSTONE_HOME+File.separatorChar+"conf"+File.separatorChar+"templates";
			}

			String newPart         = indexDir+File.separatorChar+name;
			String newConf         = newPart+File.separatorChar+"conf";
			String newIndex        = newPart+File.separatorChar+"table";
			String newLoadQueue    = newPart+File.separatorChar+"load-queue";
			String newLoadArchive  = newPart+File.separatorChar+"load-archive";
			String newExport       = newPart+File.separatorChar+"export";
			String newDic          = newPart+File.separatorChar+"dictionaries";

			File partDir = new File(newPart);
			if(partDir.exists())
			{
				System.out.println("Partition exists, exiting.");
				return;
			}
			partDir.mkdirs();


			File confDir = new File(newConf);
			confDir.mkdirs();

			copyDirectory(new File(templateDir), confDir);

			File iDir = new File(newIndex);
			iDir.mkdirs();

			File lqDir = new File(newLoadQueue);
			lqDir.mkdirs();

			File laDir = new File(newLoadArchive);
			laDir.mkdirs();

			File dicDir = new File(newDic);
			dicDir.mkdirs();

			File eDir = new File(newExport);
			eDir.mkdirs();
		}
		catch(Exception e)
		{
			e.printStackTrace();
			System.out.println("Error in the summarizer see the summarizer.log for details.");
			System.out.println(e.getMessage());
		}
	}

	public void copyDirectory(File sourceLocation , File targetLocation) throws IOException
	{

		if (sourceLocation.isDirectory())
		{
			if (!targetLocation.exists())
			{
				targetLocation.mkdir();
			}

			String[] children = sourceLocation.list();
			for (int i=0; i<children.length; i++)
			{
				copyDirectory(new File(sourceLocation, children[i]), new File(targetLocation, children[i]));
			}
		}
		else
		{

			InputStream in = new FileInputStream(sourceLocation);
			OutputStream out = new FileOutputStream(targetLocation);

			// Copy the bits from instream to outstream
			byte[] buf = new byte[1024];
			int len;
			while ((len = in.read(buf)) > 0)
			{
				out.write(buf, 0, len);
			}
			in.close();
			out.close();
		}
	}

	public boolean needsResources()
	{
		return false;
	}
}