/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.system;

import amberstone.read.Reader;

public class ReadCommand implements Command
{
	private String select;
	private String index;
	private int startYYYYMMDD;
	private int endYYYYMMDD;
	private boolean valid;
	private String args[];

	public ReadCommand()
	{

	}

	public ReadCommand(String args[])
	{
		this.args = args;

		if(args.length == 5)
		{
			if(DateRange.valid(args[3], args[4]))
			{
				valid = true;
			}
			else
			{
				System.out.println("Invalid date range.");
				valid = false;
				return;
			}
		}
		else
		{
			valid = false;
			return;
		}

		this.index = args[1];
		this.select = args[2];
		this.startYYYYMMDD = Integer.parseInt(args[3]);
		this.endYYYYMMDD = Integer.parseInt(args[4]);
	}

	public String getSelect()
	{
		return this.select;
	}

	public String[] getArgs()
	{
		return this.args;
	}

	public boolean valid()
	{
		return valid;
	}

	public String getIndex()
	{
		return this.index;
	}

	public String usage()
	{
		return "read [index ] [select] [startYYYYMMDD] [endYYYYMMDD]";
	}

	public void run(Config config)
	{
		try
		{
			Reader reader = new Reader(config);
			reader.read(startYYYYMMDD, endYYYYMMDD);
		}
		catch(Exception e)
		{
			System.out.println("Error in the reader:");
			System.out.println(e.getMessage());
			Trace trace = new Trace();
			trace.trace(e, "reader", config.getLogsHome());
		}
	}

	public boolean needsResources()
	{
		return true;
	}
}