/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.system;

import amberstone.summary.Summarizer;

public class SummarizeCommand implements Command
{
	private String config;
	public static String index;
	private int startYYYYMMDD;
	private int endYYYYMMDD;
	private int startBlock;
	private boolean valid;

	public SummarizeCommand(String args[])
	{
		if(args.length == 5 || args.length == 6)
		{
			if(DateRange.valid(args[3], args[4]))
			{
				valid = true;
			}
			else
			{
				System.out.println("Invalid date range.");
				valid = false;
				return;
			}

			this.index = args[1];
			this.config = args[2];
			this.startYYYYMMDD = Integer.parseInt(args[3]);
			this.endYYYYMMDD = Integer.parseInt(args[4]);
			if(args.length == 6)
			{
				this.startBlock = Integer.parseInt(args[6]);
			}
			else
			{
				this.startBlock = 0;
			}
		}
		else
		{
			valid = false;
		}
	}

	public boolean valid()
	{
		return valid;
	}

	public String usage()
	{
		return "aggregate [table] [config] [startYYYYMMDD] [endYYYYMMDD] <startBlock>";
	}

	public String getConfig()
	{
		return this.config;
	}

	public String getIndex()
	{
		return this.index;
	}

	public int getStartBlock()
	{
		return this.startBlock;
	}

	public int getStartYYYYMMDD()
	{
		return this.startYYYYMMDD;
	}

	public int getEndYYYYMMDD()
	{
		return this.endYYYYMMDD;
	}

	public void run(Config config)
	{
		try
		{
			Summarizer summarizer = new Summarizer(config, this.startYYYYMMDD, this.endYYYYMMDD, this.startBlock);
			summarizer.summarize();
		}
		catch(Exception e)
		{
			e.printStackTrace();
			System.out.println("Error in the summarizer see the summarizer.log for details.");
			System.out.println(e.getMessage());
		}
	}

	public boolean needsResources()
	{
		return true;
	}
}