/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.system;


public class SystemException extends Exception
{
	private boolean warning = false;

	public SystemException(String message, Exception cause)
	{
		super(message, cause);
	}

	public SystemException(String message)
	{
		super(message);
	}

	public void setWarning(boolean warning)
	{
		this.warning = warning;
	}

	public boolean getWarning()
	{
		return this.warning;
	}

	public void log(int logLevel, Logger logger)
	{
		logger.ln();
		logger.timestamp();
		logger.log("System Exception");
		logger.ln();

		if(getMessage() != null)
		{
			logger.log(getMessage());
		}

		Throwable c = getCause();

		int i = 1;

		while(c != null)
		{
			logger.ln();
			logger.log("Cause:");
			logger.log(c.getMessage());
			logger.ln();
			c = c.getCause();
			i++;
		}
	}

	public String indent(int indent)
	{
		StringBuilder buf = new StringBuilder();
		for(int i=0; i<indent; i++)
		{
			buf.append("    ");
		}

		return buf.toString();
	}
}