/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.time;

import amberstone.lang.*;

import java.util.GregorianCalendar;
import java.math.BigInteger;

import java.util.Calendar;
import java.util.List;


public class Day extends Function implements Increment
{
	private Calendar cal = new GregorianCalendar();
	private int YYYY;
	private int MM;
	private int DD;
	private DaySeries daySeries = new DaySeries();

	private static final FunctionType[][] allow = {{FunctionType.STRING_LITERAL}, {FunctionType.NUMBER_LITERAL}};

	public FunctionType[][] strict()
	{
		return allow;
	}

	public void params(Function[] params) throws ParameterException
	{
		if(params[0] instanceof NumberLiteral)
		{
			int add = ((NumberLiteral)params[0]).getInt();
			long time = System.currentTimeMillis();
			cal.setTimeInMillis(time);
			cal.add(cal.DATE, add);

			this.YYYY = cal.get(cal.YEAR);
			this.MM   = cal.get(cal.MONTH)+1;
			this.DD   = cal.get(cal.DAY_OF_MONTH);
		}
		else
		{
			try
			{
				String s = params[0].toString();
				int YYYYMMDD = DaySeries.validateAndFormat(s);
				this.YYYY = (int)(YYYYMMDD/10000);
				this.MM = (int)((YYYYMMDD % (YYYY*10000)) / 100);
				this.DD = YYYYMMDD % ((YYYY*10000)+(MM*100));
			}
			catch(Exception e)
			{
				throw new ParameterException(params[0], "Invalid day format (YYYY-MM-DD): "+params[0].toString(), e);
			}
		}
	}

	public TimeSeries getTimeSeries()
	{
		return new DaySeries(this);
	}

	public int getIncrement()
	{
		return (YYYY*10000)+(MM*100)+DD;
	}

	public String toString()
	{
		String s = this.YYYY+"-"+pad(this.MM)+"-"+pad(this.DD);
		return s;
	}

	private String pad(int i)
	{
		String s = Integer.toString(i);
		if(s.length() == 1)
		{
			return 0+s;
		}

		return s;
	}
}