/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.time;

import amberstone.lang.*;

import java.util.GregorianCalendar;
import java.math.BigInteger;
import java.util.Calendar;
import java.util.List;

public class Month extends Function implements Increment
{
	private Calendar cal = new GregorianCalendar();
	private int YYYY;
	private int MM;
	private MonthSeries monthSeries = new MonthSeries();

	private static final FunctionType[][] allow = {{FunctionType.STRING_LITERAL}, {FunctionType.NUMBER_LITERAL}};

	public FunctionType[][] strict()
	{
		return allow;
	}

	public String output(int YYYYMM)
	{
		String out = null;
		String s = Integer.toString(YYYYMM);
		out = s.substring(5,7)+"-"+s.substring(0, 4);
		return out;
	}

	public void params(Function[] params) throws ParameterException
	{
		if(params[0] instanceof NumberLiteral)
		{
			int add = ((NumberLiteral)params[0]).getInt();
			long time = System.currentTimeMillis();
			cal.setTimeInMillis(time);
			if(add > 0)
			{
				cal.add(cal.MONTH, add);
			}

			YYYY = cal.get(cal.YEAR);
			MM   = cal.get(cal.MONTH);
		}
		else
		{
			try
			{
				String s = params[0].toString();
				int YYYYMM = MonthSeries.validateAndFormat(s);
				this.YYYY = (int)(YYYYMM/100);
				this.MM = YYYYMM % (YYYY*100);
			}
			catch(Exception e)
			{
				throw new ParameterException(params[0], "Invalid month format (YYYY-MM): "+params[0].toString(), e);
			}
		}
	}

	public int getIncrement()
	{
		return (YYYY*100)+MM;
	}

	public TimeSeries getTimeSeries()
	{
		return new MonthSeries(this);
	}

	public String toString()
	{
		return YYYY+"-"+MM;
	}
}