/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.time;

import amberstone.lang.*;

import java.util.List;
import java.util.GregorianCalendar;
import java.util.Calendar;

public class Week extends Function implements Increment
{
	private Calendar cal = new GregorianCalendar();
	private int YYYY;
	private int WW;
	private WeekSeries weekSeries = new WeekSeries();

	private static final FunctionType[][] allow = {{FunctionType.STRING_LITERAL}, {FunctionType.NUMBER_LITERAL}};

	public FunctionType[][] strict()
	{
		return allow;
	}

	public void params(Function[] params) throws ParameterException
	{
		if(params[0] instanceof NumberLiteral)
		{
			int add = ((NumberLiteral)params[0]).getInt();
			long time = System.currentTimeMillis();
			cal.setTimeInMillis(time);
			if(add > 0)
			{
				cal.add(cal.WEEK_OF_YEAR, add);
			}

			YYYY = cal.get(cal.YEAR);
			WW = cal.get(cal.WEEK_OF_YEAR);
		}
		else
		{
			try
			{
				String s = params[0].toString();
				int YYYYWW = WeekSeries.validateAndFormat(s);
				this.YYYY = YYYYWW/100;
				this.WW = YYYYWW%YYYY;
			}
			catch(Exception e)
			{
				throw new ParameterException(params[0], "Invalid week format (YYYY-WW): "+params[0].toString(), e);
			}
		}
	}

	public TimeSeries getTimeSeries()
	{
		return new WeekSeries(this);
	}

	public int getIncrement()
	{
		return (YYYY*100)+WW;
	}

	public String toString()
	{
		return YYYY+"-"+WW;
	}
}