/**
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
**/

package amberstone.transaction;

import amberstone.lang.*;
import amberstone.data.*;

import java.util.*;

public class SetProp extends Function implements Prop
{
	private KeyFunction keyFunc;
	private BooleanFunction filter;
	private int size;
	private Key name;
	private List data = new ArrayList();
	private boolean sort;
	private Comparator keySort = new KeySort();

	private static final FunctionType[][] allow = {{FunctionType.KEY_FUNCTION, FunctionType.NUMBER_LITERAL},
	                                               {FunctionType.KEY_FUNCTION, FunctionType.NUMBER_LITERAL, FunctionType.BOOLEAN_LITERAL, FunctionType.BOOLEAN_LITERAL}};

	public SetProp()
	{

	}

	public SetProp(Key name, KeyFunction keyFunc, int size, boolean sort, BooleanFunction filter)
	{
		this.sort = sort;
		this.name = name;
		this.keyFunc = keyFunc;
		this.size = size;
		this.filter = filter;
	}

	public FunctionType[][] strict()
	{
		return allow;
	}

	public void params(Function[] params)
	{
		this.keyFunc = (KeyFunction)params[0];
		this.size    = ((NumberLiteral)params[1]).getInt();

		if(params.length == 4)
		{
			this.sort = ((BooleanLiteral)params[2]).getBoolean();
			this.filter = (BooleanFunction) params[3];
		}
		else
		{
			this.sort = false;
			this.filter = new BooleanLiteral(true);
		}
	}

	public Prop cloneProp(Key _name)
	{
		return new SetProp(_name, this.keyFunc, this.size, this.sort, this.filter);
	}

	public void update() throws FunctionException
	{
		Key k = keyFunc.getKey();
		int s = data.size();
		if(!k.isNull() && filter.getBoolean() && s < size)
		{
			for(int i=0; i<s; i++)
			{
				Key ak = (Key)data.get(i);
				if(ak.equalsKey(k))
				{
					return;
				}
			}
			data.add(k.cloneKey());
		}
	}

	public int getInt()
	{
		return data.size();
	}

	public long getLong()
	{
		return data.size();
	}

	public Key getName()
	{
		return this.name;
	}

	public Key getKey()
	{
		if(sort)
		{
			Collections.sort(data, keySort);
		}

		CompoundKey ckey = new CompoundKey(size);

		for(int i=0; i<data.size(); i++)
		{
			ckey.append((KeyFunction)data.get(i));
		}

		ckey.DELIM = "~";
		ckey.clone = false;
		return ckey;
	}
}